/*==========================================================
 Copyright (C) 2005 YOKOGAWA ELECTRIC CORPORATION

    ALL RIGHTS RESERVED BY YOKOGAWA ELECTRIC CORPORATION.
THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT
WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY
AND/OR FITNESS FOR A PARTICULAR PURPOSE.

                            YOKOGAWA ELECTRIC CORPORATION
==========================================================*/
// CommDialog.cpp

#include "stdafx.h"
#include "WT1600Demo.h"
#include "CommDialog.h"
#include "MainDialog.h"
#include "Connection.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

const CString GPIB_ADDR[] = {"1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16",
                             "17","18","19","20","21","22","23","24","25","26","27","28","29","30"};
const CString RS232_PORT[] = {"COM1","COM2","COM3","COM4"};
const CString RS232_BAUD[] = {"1200","2400","4800","9600","19200"};

/////////////////////////////////////////////////////////////////////////////
// CCommDialog

CCommDialog::CCommDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CCommDialog::IDD, pParent)
{
	//m_connection = NULL;
	//{{AFX_DATA_INIT(CCommDialog)
	m_sIp = _T("");
	m_sPwd = _T("");
	m_sUser = _T("");
	m_iWire = -1;
	//}}AFX_DATA_INIT
}


void CCommDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCommDialog)
	DDX_Control(pDX, IDC_COMBO_HDSHK, m_comboHdshk);
	DDX_Control(pDX, IDC_COMBO_TMNTR, m_comboTerminator);
	DDX_Control(pDX, IDC_COMBO_FORMAT, m_comboFormat);
	DDX_Control(pDX, IDC_EDIT_USER, m_editUser);
	DDX_Control(pDX, IDC_EDIT_PWD, m_editPwd);
	DDX_Control(pDX, IDC_EDIT_IP, m_editIp);
	DDX_Control(pDX, IDC_COMBO_PORT, m_comboPort);
	DDX_Control(pDX, IDC_COMBO_BAUD, m_comboBaud);
	DDX_Control(pDX, IDC_COMBO_ADDR, m_comboAddr);
	DDX_Text(pDX, IDC_EDIT_IP, m_sIp);
	DDX_Text(pDX, IDC_EDIT_PWD, m_sPwd);
	DDX_Text(pDX, IDC_EDIT_USER, m_sUser);
	DDX_Radio(pDX, IDC_RADIO_GPIB, m_iWire);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CCommDialog, CDialog)
	//{{AFX_MSG_MAP(CCommDialog)
	ON_BN_CLICKED(IDC_RADIO_GPIB, OnRadioGpib)
	ON_BN_CLICKED(IDC_RADIO_RS232, OnRadioRs232)
	ON_BN_CLICKED(IDC_RADIO_ETHER, OnRadioEther)
	ON_EN_CHANGE(IDC_EDIT_USER, OnChangeEditUser)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCommDialog message handler

BOOL CCommDialog::OnInitDialog()
{
    m_hIcon = AfxGetApp()->LoadIcon(IDI_ICON);
	CDialog::OnInitDialog();
    //SetIcon(m_hIcon, FALSE);
    SetIcon(m_hIcon, TRUE);

    //set the list items into comboBoxs
	int i; 
	for (i=0; i<30; i++)
	{
		m_comboAddr.AddString(GPIB_ADDR[i]);
	}
	for (i=0; i<4; i++)
	{
		m_comboPort.AddString(RS232_PORT[i]);
	}
	for (i=0; i<5; i++)
	{
		m_comboBaud.AddString(RS232_BAUD[i]);
	}

	// set the default values of comboBox
	m_comboAddr.SetCurSel(0);
	m_comboPort.SetCurSel(0);
	m_comboBaud.SetCurSel(0);
	m_comboHdshk.SetCurSel(0);
	m_comboFormat.SetCurSel(0);
	m_comboTerminator.SetCurSel(0);

    m_sUser = "anonymous";
    UpdateData(false);
	// set the default selection as GPIB
	CButton* pBtn = (CButton*)GetDlgItem(IDC_RADIO_GPIB);
	pBtn->SetCheck(1);

    // Call GPIB RadioButton's event to refresh display.
	OnRadioGpib(); 
	return true;
}

void CCommDialog::OnChangeEditUser() 
{
    UpdateData(true);
    //when input anonymous or null, disable password.
    if (m_sUser == "anonymous" || m_sUser == "")
    {
        m_editPwd.SetReadOnly(true);
    }
    else
    {
        m_editPwd.SetReadOnly(false);
    }
}

void CCommDialog::OnRadioGpib() 
{
    //set GPIB options enable.
	m_comboAddr.EnableWindow(true);
    //set EtherNet options disable.
    m_editIp.EnableWindow(false);
	m_editUser.EnableWindow(false);
	m_editPwd.EnableWindow(false);
    //set RS232 options disable.
	m_comboPort.EnableWindow(false);
	m_comboBaud.EnableWindow(false);
}

void CCommDialog::OnRadioEther() 
{
    //set GPIB options disable.
	m_comboAddr.EnableWindow(false);
    //set EtherNet options enable.
    m_editIp.EnableWindow(true);
	m_editUser.EnableWindow(true);
	m_editPwd.EnableWindow(true);
    //set RS232 options disable.
	m_comboPort.EnableWindow(false);
	m_comboBaud.EnableWindow(false);
    OnChangeEditUser();
}

void CCommDialog::OnRadioRs232() 
{
    //set GPIB options disable.
	m_comboAddr.EnableWindow(false);
    //set EtherNet options disable.
    m_editIp.EnableWindow(false);
	m_editUser.EnableWindow(false);
	m_editPwd.EnableWindow(false);
    //set RS232 options enable.
	m_comboPort.EnableWindow(true);
	m_comboBaud.EnableWindow(true);
}

void CCommDialog::OnOK() 
{
	CString address;
	// get the checked one
	UINT nID = GetCheckedRadioButton(IDC_RADIO_GPIB,IDC_RADIO_RS232);

    int tryAgain = IDOK;//when connection failed, ask for trying again.
    while(tryAgain == IDOK)
    {
        UpdateData();//used for DDX refresh
	    switch (m_iWire)
	    {
		    case 0: // when GPIB was selected, set options into m_sAddress.
			    m_comboAddr.GetLBText(m_comboAddr.GetCurSel(),address);
			    m_connection.m_sAddress = address;
			    m_connection.m_iPort = 1;
			    break;

		    case 1: // when EtherNet was selected, set options into m_sAddress.
                //do not set user/passwd when no user name.
			    if (m_sUser != "anonymous" && m_sUser != "")
				    m_connection.m_sAddress = m_sIp + "," + m_sUser + "," + m_sPwd;
			    else
				    m_connection.m_sAddress = m_sIp + "," + "anonymous,";
			    m_connection.m_iPort = 4;
			    break;

		    case 2: // when RS232 was selected, set options into m_sAddress.
			    address.Format(_T("%d"),m_comboPort.GetCurSel()+1);
			    m_connection.m_sAddress = address;
			    address.Format(_T("%d"),m_comboBaud.GetCurSel());
			    m_connection.m_sAddress = m_connection.m_sAddress +"," + address + ",0,2";
			    m_connection.m_iPort = 2;
			    break;
	    }
	    // connect to the instrument
	    if (m_connection.Initialize() == 0)
        {
	        CDialog::OnOK();
            return;
        }
        //display the asking dialog.
        tryAgain = MessageBox("Can not connect with the instrument, try again?", "Connection failed.", MB_OKCANCEL|MB_ICONERROR);
    }
}
