/*==========================================================
 Copyright (C) 2005 YOKOGAWA ELECTRIC CORPORATION

    ALL RIGHTS RESERVED BY YOKOGAWA ELECTRIC CORPORATION.
THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT
WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY
AND/OR FITNESS FOR A PARTICULAR PURPOSE.

                            YOKOGAWA ELECTRIC CORPORATION
==========================================================*/
// MainDialog.cpp

#include "stdafx.h"
#include "WT1600Demo.h"
#include "MainDialog.h"
#include "CommDialog.h"
#include "Connection.h"
#include <fstream.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//##########################################//
// CMainDialog
//##########################################//
const int MAX_ITEM = 10;      //the max items count.
const int MAX_ELEMENT = 6;    //the max elements count.
const int MAX_LINES = 499;    //the max monitor lines count.
const int TIMER_1 = 1;        //used when getting data by update rate.
const int TIMER_2 = 2;        //used when getting data by timer.
const char* MODEL = "760101"; //used when testing instrument model.

CMainDialog::CMainDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CMainDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMainDialog)
	m_sEditTest = _T("");
	m_sEditSend = _T("");
	m_sEditRcv = _T("");
	m_sEditError = _T("");
	m_sComboItems = _T("");
	m_sFileName = _T("");
	m_iRadioElement = -1;
	//}}AFX_DATA_INIT
}

void CMainDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMainDialog)
	DDX_Control(pDX, IDC_COMBOR, m_comboR);
	DDX_Control(pDX, IDC_COMBOV, m_comboU);
	DDX_Control(pDX, IDC_COMBOC, m_comboI);
	DDX_Text(pDX, IDC_EDIT_TEST, m_sEditTest);
	DDX_Text(pDX, IDC_EDIT_SEND, m_sEditSend);
	DDX_Text(pDX, IDC_EDIT_RCV, m_sEditRcv);
	DDX_Text(pDX, IDC_EDIT_ERR, m_sEditError);
	DDX_CBString(pDX, IDC_COMBO_ITEMS, m_sComboItems);
	DDX_Text(pDX, IDC_EDIT_FILE, m_sFileName);
	DDX_Radio(pDX, IDC_RADIO_E1, m_iRadioElement);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CMainDialog, CDialog)
	//{{AFX_MSG_MAP(CMainDialog)
	ON_BN_CLICKED(IDC_RADIO_E1, OnRadioE1)
	ON_BN_CLICKED(IDC_RADIO_E2, OnRadioE2)
	ON_BN_CLICKED(IDC_RADIO_E6, OnRadioE6)
	ON_BN_CLICKED(IDC_RADIO_E3, OnRadioE3)
	ON_BN_CLICKED(IDC_RADIO_E4, OnRadioE4)
	ON_BN_CLICKED(IDC_RADIO_E5, OnRadioE5)
	ON_BN_CLICKED(IDC_BUTTON_SETRANGE, OnButtonSetRange)
	ON_BN_CLICKED(IDC_BUTTON_SENDCMD, OnButtonSendCmd)
	ON_BN_CLICKED(IDC_RADIO_NML, OnRadioNml)
	ON_CBN_SELCHANGE(IDC_COMBO_ITEMS, OnSelchangeComboItems)
	ON_BN_CLICKED(IDC_RADIO_HAR, OnRadioHar)
	ON_BN_CLICKED(IDC_BUTTON_GETITEMS, OnButtonGetItems)
	ON_BN_CLICKED(IDC_BUTTON_SETRATE, OnButtonSetRate)
	ON_BN_CLICKED(IDC_BUTTON_SETITEMS, OnButtonSetItems)
	ON_BN_CLICKED(IDC_BUTTON_GETDATA_S, OnButtonGetDataS)
	ON_BN_CLICKED(IDC_BUTTON_GETDATA_R, OnButtonGetDataR)
	ON_BN_CLICKED(IDC_BUTTON_GETDATA_T, OnButtonGetDataT)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_CHECK_SAVE, OnCheckSave)
	ON_BN_CLICKED(IDC_BUTTON_SCLR, OnButtonSclr)
	ON_BN_CLICKED(IDC_BUTTON_RCLR, OnButtonRclr)
	ON_BN_CLICKED(IDC_BUTTON_GETERR, OnButtonGetError)
	ON_BN_CLICKED(IDC_BUTTON_HEADER, OnButtonHeader)
	ON_BN_CLICKED(IDC_BUTTON_RST, OnButtonRst)
	ON_BN_CLICKED(IDCLOSE, OnDlgClose)
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_BUTTON_VERBOSE, OnButtonVerbose)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//==========================================//
///<summary>
///Function: ConstInit
///</summary>
//==========================================//
void CMainDialog::InitListItem()
{
    //===========================
    ///Error Code from Tmctl
    //===========================
    //Error Number  Problem                                  Solutions
	errorMsg[0]  = "Device not found";            //   2  1 Check the wiring.  
	errorMsg[1]  = "Connection to device failed"; //   4  2 Check the wiring.  
	errorMsg[2]  = "Device not connected";        //   8  3 Connect the device using the initialization function.  
	errorMsg[3]  = "Device already connected";    //  16  4 Two connections cannot be opened.  
	errorMsg[4]  = "Incompatible PC";             //  32  5 Check the hardware you are using.  
	errorMsg[5]  = "Illegal parameter";           //  64  6 Check parameter type etc. 
    errorMsg[6].Empty();
	errorMsg[7]  = "Send error";                  // 256  8 Check the wiring, address, and ID.
	errorMsg[8]  = "Receive error";               // 512  9 Check whether an error occurred on the device.  
	errorMsg[9]  = "Received data not block data";//1024 10
	errorMsg[10] = "System error";                //4096 11 There is a problem with the operating environment.  
	errorMsg[11] = "Illegal device ID";           //8192 12 Use the ID of the device acquired by the initialization function.  

    //AutoRange item
    rangeListAuto = "AUTO";
	Direct = "DIRECT";
	Sensor = "SENSOR";
    //===========================
    ///UpdateRate List
    //===========================
    updateRateList[0] = "50ms";
    updateRateList[1] = "100ms";
    updateRateList[2] = "200ms";
    updateRateList[3] = "500ms";
    updateRateList[4] = "1s";
    updateRateList[5] = "2s";
    updateRateList[6] = "5s";
    updateRateList[7].Empty();

    //===========================
    ///Voltage Range List
    //===========================
    //CrestFactor = 3
    voltageList[0][0] = "1.5V";
    voltageList[0][1] = "3V";
    voltageList[0][2] = "6V";
    voltageList[0][3] = "10V";
    voltageList[0][4] = "15V";
    voltageList[0][5] = "30V";
    voltageList[0][6] = "60V";
    voltageList[0][7] = "100V";
    voltageList[0][8] = "150V";
    voltageList[0][9] = "300V";
    voltageList[0][10] = "600V";
    voltageList[0][11] = "1000V";
    voltageList[0][12].Empty();
    //CrestFactor = 6
    voltageList[1][0] = "0.75V";
    voltageList[1][1] = "1.5V";
    voltageList[1][2] = "3V";
    voltageList[1][3] = "5V";
    voltageList[1][4] = "7.5V";
    voltageList[1][5] = "15V";
    voltageList[1][6] = "30V";
    voltageList[1][7] = "50V";
    voltageList[1][8] = "75V";
    voltageList[1][9] = "150V";
    voltageList[1][10] = "300V";
    voltageList[1][11] = "500V";
    voltageList[1][12] .Empty();

    //===========================
    ///Current Range List
    //===========================
    //CrestFactor = 3, Direct
    currentList[0][0] = "10mA";
    currentList[0][1] = "20mA";
    currentList[0][2] = "50mA";
    currentList[0][3] = "100mA";
    currentList[0][4] = "200mA";
    currentList[0][5] = "500mA";
    currentList[0][6] = "1A";
    currentList[0][7] = "2A";
    currentList[0][8] = "5A";
    currentList[0][9] = "10A";
    currentList[0][10] = "20A";
    currentList[0][11] = "50A";
    currentList[0][12].Empty();

    //CrestFactor = 6, Direct
    currentList[1][0] = "5mA";
    currentList[1][1] = "10mA";
    currentList[1][2] = "25mA";
    currentList[1][3] = "50mA";
    currentList[1][4] = "100mA";
    currentList[1][5] = "250mA";
    currentList[1][6] = "500mA";
    currentList[1][7] = "1A";
    currentList[1][8] = "2.5A";
    currentList[1][9] = "5A";
    currentList[1][10] = "10A";
    currentList[1][11] = "25A";
    currentList[1][12].Empty();

    //CrestFactor = 3, Sensor
    currentList[2][0] = "50mV";
    currentList[2][1] = "100mV";
    currentList[2][2] = "250mV";
    currentList[2][3] = "500mV";
    currentList[2][4] = "1V";
    currentList[2][5] = "2.5V";
    currentList[2][6] = "5V";
    currentList[2][7] = "10V";
    currentList[2][8].Empty();
    currentList[2][9].Empty();
    currentList[2][10].Empty();
    currentList[2][11].Empty();
    currentList[2][12].Empty();

    //CrestFactor = 6, Sensor
    currentList[3][0] = "25mV";
    currentList[3][1] = "50mV";
    currentList[3][2] = "125mV";
    currentList[3][3] = "250mV";
    currentList[3][4] = "500mV";
    currentList[3][5] = "1.25V";
    currentList[3][6] = "2.5V";
    currentList[3][7] = "5V";
    currentList[3][8].Empty();
    currentList[3][9].Empty();
    currentList[3][10].Empty();
    currentList[3][11].Empty();
    currentList[3][12].Empty();

    //===========================
    ///Element List
    //===========================
    eList[0] = "1";
    eList[1] = "2";
    eList[2] = "3";
    eList[3] = "4";
    eList[4] = "5";
    eList[5] = "6";
    eList[6] = "SGMA";
    eList[7] = "SGMB";
    eList[8] = "SGMC";
    eList[9].Empty();

    //===========================
    ///Order List
    //===========================
    oList[0] = "TOTAL";
    oList[1] = "DC";
    oList[2] = "1";
    oList[3] = "2";
    oList[4] = "3";
    oList[5] = "4";
    oList[6] = "5";
    oList[7] = "6";
    oList[8] = "7";
    oList[9] = "8";
    oList[10] = "9";
    oList[11] = "10";
    oList[12] = "11";
    oList[13] = "12";
    oList[14] = "13";
    oList[15] = "14";
    oList[16] = "15";
    oList[17] = "16";
    oList[18] = "17";
    oList[19] = "18";
    oList[20] = "19";
    oList[21] = "20";
    oList[22] = "21";
    oList[23] = "22";
    oList[24] = "23";
    oList[25] = "24";
    oList[26] = "25";
    oList[27] = "26";
    oList[28] = "27";
    oList[29] = "28";
    oList[30] = "29";
    oList[31] = "30";
    oList[32] = "31";
    oList[33] = "32";
    oList[34] = "33";
    oList[35] = "34";
    oList[36] = "35";
    oList[37] = "36";
    oList[38] = "37";
    oList[39] = "38";
    oList[40] = "39";
    oList[41] = "40";
    oList[42] = "41";
    oList[43] = "42";
    oList[44] = "43";
    oList[45] = "44";
    oList[46] = "45";
    oList[47] = "46";
    oList[48] = "47";
    oList[49] = "48";
    oList[50] = "49";
    oList[51] = "50";
    oList[52] = "51";
    oList[53] = "52";
    oList[54] = "53";
    oList[55] = "54";
    oList[56] = "55";
    oList[57] = "56";
    oList[58] = "57";
    oList[59] = "58";
    oList[60] = "59";
    oList[61] = "60";
    oList[62] = "61";
    oList[63] = "62";
    oList[64] = "63";
    oList[65] = "64";
    oList[66] = "65";
    oList[67] = "66";
    oList[68] = "67";
    oList[69] = "68";
    oList[70] = "69";
    oList[71] = "70";
    oList[72] = "71";
    oList[73] = "72";
    oList[74] = "73";
    oList[75] = "74";
    oList[76] = "75";
    oList[77] = "76";
    oList[78] = "77";
    oList[79] = "78";
    oList[80] = "79";
    oList[81] = "80";
    oList[82] = "81";
    oList[83] = "82";
    oList[84] = "83";
    oList[85] = "84";
    oList[86] = "85";
    oList[87] = "86";
    oList[88] = "87";
    oList[89] = "88";
    oList[90] = "89";
    oList[91] = "90";
    oList[92] = "91";
    oList[93] = "92";
    oList[94] = "93";
    oList[95] = "94";
    oList[96] = "95";
    oList[97] = "96";
    oList[98] = "97";
    oList[99] = "98";
    oList[100] = "99";
    oList[101] = "100";
    oList[102].Empty();

    //==============================
    ///Normal Data Function List
    //==============================
    ///to get more about the command
    ///of a function, see 5.5 in the
    ///communication manual.
    //==============================
    nList[0] = "URMS"; //"Urms"
    nList[1] = "UMN"; //"Umean"
    nList[2] = "UDC"; //"Udc"
    nList[3] = "UAC"; //"Uac"
    nList[4] = "IRMS"; //"Irms"
    nList[5] = "IMN"; //"Imean"
    nList[6] = "IDC"; //"Idc"
    nList[7] = "IAC"; //"Iac"
    nList[8] = "P";
    nList[9] = "S";
    nList[10] = "Q";
    nList[11] = "LAMBDA"; //""
    nList[12] = "PHI"; //""
    nList[13] = "FU"; //"FreqU"
    nList[14] = "FI"; //"FreqI"
    nList[15] = "UPPEAK"; //"U +peak"
    nList[16] = "UMPEAK"; //"U -peak"
    nList[17] = "IPPEAK"; //"i +peak"
    nList[18] = "IMPEAK"; //"i -peak"
    nList[19] = "CFU"; //"CfU"
    nList[20] = "CFI"; //"CfI"
    nList[21] = "FFU"; //"FfU"
    nList[22] = "FFI"; //"FfI"
    nList[23] = "Z";
    nList[24] = "RS"; //"Rs"
    nList[25] = "XS"; //"Xs"
    nList[26] = "RP"; //"Rp"
    nList[27] = "XP"; //"Xp"
    nList[28] = "PC"; //"Pc"
    nList[29] = "TIME"; //"i -Time"
    nList[30] = "WH"; //"Wp"
    nList[31] = "WHP"; //"Wp+"
    nList[32] = "WHM"; //"Wp-"
    nList[33] = "AH"; //"q"
    nList[34] = "AHP"; //"q+"
    nList[35] = "AHM"; //"q-"
    //NO ELEMENT SETTINGS BELOW
    nList[36] = "ETA"; //""
    nList[37] = "SETA"; //"1/"
    //(Motor)
    nList[38] = "PM"; //"Pm"
    nList[39] = "TORQUE"; //"Torque"
    nList[40] = "SPEED"; //"Speed"
    nList[41] = "SLIP"; //"Slip"
    nList[42] = "SYNC"; //"Sync"
    nList[43] = "MAETA"; //"mA"
    nList[44] = "MBETA"; //"mB"
    //(1~4)
    nList[45] = "DURMS"; //"Urms"
    nList[46] = "DUMN"; //"Umn"
    nList[47] = "DUDC"; //"Udc"
    nList[48] = "DUAC"; //"Uac"
    nList[49] = "DIRMS"; //"Irms"
    nList[50] = "DIMN"; //"Imn"
    nList[51] = "DIDC"; //"Idc"
    nList[52] = "DIAC"; //"Iac"
    nList[53].Empty();

    //============================
    ///Harmonics Data Function List
    //============================
    //E1~E9,ORDER
    hList[0] = "U";
    hList[1] = "I";
    hList[2] = "P";
    hList[3] = "S";
    hList[4] = "Q";
    hList[5] = "LAMBDA"; //""
    hList[6] = "PHI"; //""
    hList[7] = "PHIU"; //"U"
    hList[8] = "PHII"; //"I"
    hList[9] = "Z";
    hList[10] = "RS"; //"Rs"
    hList[11] = "XS"; //"Xs"
    hList[12] = "RP"; //"Rp"
    hList[13] = "XP"; //"Xp"
    hList[14] = "UHDF"; //"Uhdf"
    hList[15] = "IHDF"; //"Ihdf"
    hList[16] = "PHDF"; //"Phdf"
    //NO ORDER SETTINGS 
    hList[17] = "UTHD"; //"Uthd"
    hList[18] = "ITHD"; //"Ithd"
    hList[19] = "PTHD"; //"Pthd"
    hList[20] = "UTHF"; //"Uthf"
    hList[21] = "ITHF"; //"Ithf"
    hList[22] = "UTIF"; //"Utif"
    hList[23] = "ITIF"; //"Itif"
    hList[24] = "HVF"; //"Hvf"
    hList[25] = "HCF"; //"Hcf"
    hList[26] = "FU"; //"fU"
    hList[27] = "FI"; //"fI"
    //E7~E9
    hList[28] = "PHI_U1U2"; //"U1-U2"
    hList[29] = "PHI_U1U3"; //"U1-U3"
    hList[30] = "PHI_U1I1"; //"U1-I1"
    hList[31] = "PHI_U1I2"; //"U1-I2"
    hList[32] = "PHI_U1I3"; //"U1-I3"
    //CUSTEM FUNCTIONS
    hList[33] = "F1"; //"F1"
    hList[34] = "F2"; //"F2"
    hList[35] = "F3"; //"F3"
    hList[36] = "F4"; //"F4"
    hList[37].Empty();
}

//==========================================//
///Dialog Init
//==========================================//
BOOL CMainDialog::OnInitDialog() 
{
    m_hIcon = AfxGetApp()->LoadIcon(IDI_ICON);
	CDialog::OnInitDialog();
    SetIcon(m_hIcon, TRUE);
    //SetIcon(m_hIcon, FALSE);

    InitListItem();

    CWnd* pWnd;
    CString model = CConnection::m_sName;
    model = CutLeft((char)10, &model);

    //----------------------#set ModelType display#
    pWnd = GetDlgItem(IDC_MODEL_EDIT);
    pWnd->SetWindowText(model);

    //check model.
    CutLeft(',' , &model);
    //model = model.Mid(model.Find(",") + 1);
    int symbol = model.Find("-");
    if (model.Left(symbol) != MODEL)
    {
        DispError("it seems not WT1600, program may run incorrectly!");
    }

    //----------#set unuse element RadioBox disable#
    lastElement = atoi(model.Mid(symbol+1, 1)) + atoi(model.Mid(symbol + 2, 1));
    int elementIndex;
    for (elementIndex = lastElement; elementIndex < MAX_ELEMENT; elementIndex ++)
    {
		pWnd = GetDlgItem(IDC_RADIO_E1 + elementIndex);
        pWnd->EnableWindow(false);
    }
    //----------------------#Queries the Model(eql to CConnection::m_sName)#
    CString value;
	CString msg = "*IDN?";
    SetSendMonitor(msg);
    if (!QueriesData(30, msg, &value))
    {
        return false;
    }
    model = CutLeft((char)10, &value);//cut left with LF.
    SetReceiveMonitor(model);

    //----------------------#Queries the CrestFactor#
    msg = ":INPUT:CFACTOR?";
    SetSendMonitor(msg);
    if (!QueriesData(20, msg, &value))
    {
        return false;
    }
    crestFactor = CutLeft((char)10, &value);//cut left with LF.
    SetReceiveMonitor(crestFactor);

    //set the first RadioBox to be checked.
    if (lastElement >0)
    {
        CheckDlgButton(IDC_RADIO_E1, BST_CHECKED);
    }
    //get current item settings from instrument.
    ReadItemSettings(0);
    OnRadioE1();
    
    pWnd = GetDlgItem(IDC_EDIT_FILE);
    pWnd->SetWindowText("MeasuredData.csv");
    pWnd->EnableWindow(false);
    pWnd = GetDlgItem(IDC_EDIT_TMR);
    pWnd->SetWindowText("500");
    return TRUE;
}

//==========================================//
///Dialog Close
//==========================================//
void CMainDialog::OnClose()
{
	//close connection when exit window by system menu.
	m_connection.Finish();
	CDialog::OnClose();
}
void CMainDialog::OnDlgClose()
{
	//close connection when exit window by system menu.
	m_connection.Finish();
    CMainDialog::OnOK();
}

//##########################################//
// COMMAN FUNCTIONS
//##########################################//

//==========================================//
///<summary>Function: ComboListSel</summary>
///<remark>
///to get and display the given item in the list
///</remark>
//==========================================//
void CMainDialog::ComboListSel(CComboBox* pCombo, CString content)
{
    int n;
    CString listText;
    for (n = 0; n < pCombo->GetCount(); n ++)
    {
        pCombo->GetLBText(n, listText);
        if (listText == content)
        {
            pCombo->SetCurSel(n);
            break;
        }
    }
}

//==========================================//
/// <summary> Function: CutLeft </summary>
/// <remarks>
///cut the left half to outData,
///and the right portion remain in inData.
/// </remarks>
/// <example>
///symbol:"2", in:"12345" => out:"1", in:"345"
/// </example>
//==========================================//
CString CMainDialog::CutLeft(char symbol, CString* inData)
{
    CString outData = *inData;
    int pos = (*inData).Find(symbol);
    if (pos == -1)
    {
        //if no symbol, cut with LF.
        pos = (*inData).Find(10);
    }
    if (pos != -1)
    {
        outData = (*inData).Left(pos);
        *inData = (*inData).Mid(pos + 1);
    }

    //cut data when harmonics mode
    pos = outData.Find(" ");
    if (pos != -1)
    {
        outData = outData.Mid(pos + 1);
    }
    return outData;
}

//==========================================//
///<summary>Function: QueriesData</summary>
//==========================================//
bool CMainDialog::QueriesData(int maxLength, CString sMsg, CString* data)
{
    int rtn;
	//Send Command.
    rtn = m_connection.Send(sMsg);
    if (rtn != 0)
    {
	    DispError(m_connection.GetLastError());
	    return false;
    }

	//Queries Data.
    int realLength;
    char* buf;
    if ((buf = new char[maxLength]) == NULL)
    {
        return false;
    }
    memset(buf,0,maxLength);
    rtn = m_connection.Receive(buf, maxLength, &realLength);
    (*data).Format(_T("%s"), buf);
    delete[] buf;
    if (rtn != 0)
    {
	    DispError(m_connection.GetLastError());
	    return false;
    }
	return true;
}

//==========================================//
///<summary>Function: SetSendMonitor</summary>
//==========================================//
void CMainDialog::SetSendMonitor(CString msg)
{
    CEdit* pEdit = (CEdit*)GetDlgItem(IDC_EDIT_SEND);
    CString sendMsg;
    pEdit->GetWindowText(sendMsg);
    //add return symbol into msg.
    if (sendMsg != "")
    {
        sendMsg += "\r\n";
    }
    sendMsg += msg;

    //get the count of rows(return symbol),
    //cut off the old row when rows count > 500.
    int lineCount = 0;
    int lF = 0;
    while (sendMsg.Find(0x0A, lF) != -1)
    {
        lF = sendMsg.Find(0x0A, lF) + 1;
        lineCount ++;
    }
    if (lineCount > MAX_LINES)
    {
        CutLeft((char)10, &sendMsg);
    }
    //set to display.
    pEdit->SetWindowText(sendMsg);
    pEdit->LineScroll(lineCount);
}
//==========================================//
/// <summary>
/// Function: SetReceiveMonitor
/// </summary>
//==========================================//
void CMainDialog::SetReceiveMonitor(CString data)
{
    CEdit* pEdit = (CEdit*)GetDlgItem(IDC_EDIT_RCV);
    CString rcvData;
    pEdit->GetWindowText(rcvData);
    //cut off the terminator into data.
    data.Replace(10, 0);
    data.Format(_T("%s"), (LPSTR)(LPCTSTR)data);
    //add return in data.
    if (rcvData != "")
    {
        rcvData += "\r\n";
    }
    rcvData += data;

    //get the count of return symbol,
    //cut off the old row when rows count > 500.
    int lineCount = 0;
    int lF = 0;
    while (rcvData.Find(0x0A, lF) != -1)
    {
        lF = rcvData.Find(0x0A, lF) + 1;
        lineCount ++;
    }
    if (lineCount > MAX_LINES)
    {
        CutLeft((char)10, &rcvData);
    }
    //set to display.
    pEdit->SetWindowText(rcvData);
    pEdit->LineScroll(lineCount);
}
//==========================================//
/// <summary> Function: DispError </summary>
///<remark>
///the errorID is received from tmctl.
///get the errorMsg corresponding the errorID.
///</remark>
//==========================================//
void CMainDialog::DispError(int errorID)
{
    CWnd* pWnd = GetDlgItem(IDC_EDIT_ERR);
    if (errorID == 0)
    {
        pWnd->SetWindowText("getting detail error failed.");
        return;
    }
    int n = 0;
    while (2 << n != errorID)
    {
        n ++;
    }
    //set errorMsg to display.
    pWnd->SetWindowText(errorMsg[n]);
}
void CMainDialog::DispError(CString errorInfo)
{
    //set errorMsg to display.
    CWnd* pWnd = GetDlgItem(IDC_EDIT_ERR);
    pWnd->SetWindowText(errorInfo);    
}

//==========================================//
/// <summary> Function: GetData </summary>
//==========================================//
void CMainDialog::GetItemData()
{
    int n;
    int rtn;
    CString msg;

    UpdateData();
    //----------------------#print title(Function) when getting data once#
    CWnd* pWnd = GetDlgItem(IDC_BUTTON_GETDATA_S);
    if (IsDlgButtonChecked(IDC_CHECK_SAVE) 
        && !m_sFileName.IsEmpty()//when file name is null, do not save.
        && pWnd->IsWindowEnabled())
    {
        //print titles(function) when saveBox is checked.
        ofstream csvFile(m_sFileName, ios::ate);
        if (csvFile.fail())
        {
            MessageBox("file can not open!", "Error", MB_ICONERROR | MB_OK);
            return;
        }

        //out put current time.
        SYSTEMTIME CurTime;
        GetLocalTime(&CurTime);
        csvFile << "Date: " << CurTime.wYear << "/" << CurTime.wMonth;
        csvFile << "/" << CurTime.wDay << ",";

        int fLoop = IDC_COMBO_F1;
        CString function;
        CComboBox* pCombo;
        for (n = 0; n < atoi(m_sComboItems); n ++)
        {
	        pCombo = (CComboBox*)GetDlgItem(fLoop ++);
            pCombo->GetWindowText(function);
            csvFile << function << ",";
        }
        csvFile << "\n";
        csvFile.close();
    }

    //----------------------#get data#
    if (IsDlgButtonChecked(IDC_RADIO_NML))
    {
        msg = ":NUMERIC:NORMAL:VALUE?";
    }
    else
    {
        msg = ":NUMERIC:HARMONICS:VALUE?";
    }

    //----------------------#send message#
    SetSendMonitor(msg);
    //###ASCII:TmcSend(); FLOAT:TmcSendBuLength()###
    if (IsDlgButtonChecked(IDC_RADIO_ASC))
    {
        rtn = m_connection.Send(msg);
    }
    else
    {
        rtn = m_connection.SendByLength(msg, msg.GetLength());
    }
    if (rtn != 0)
    {
	    DispError(m_connection.GetLastError());
	    return;
    }

    //----------------------#receive values#
    char* buf;
    int maxLength;
    int realLength;
    CString value;

    CEdit* pEdit;
    int dLoop = IDC_EDIT_D1;
    CString data;

    if (IsDlgButtonChecked(IDC_RADIO_ASC))
    {
        //----------------------#receive values by ASCII#
        //###ASCII:TmcReceive()###
        UpdateData(true);
        maxLength = 15 * atoi(m_sComboItems);
        if ((buf = new char[maxLength]) == NULL)
        {
            return;
        }
        memset(buf,0,maxLength);
        rtn = m_connection.Receive(buf, maxLength, &realLength);
        value.Format(_T("%s"), buf);
        delete[] buf;
        if (rtn != 0)
        {
	        DispError(m_connection.GetLastError());
	        return;
        }
        SetReceiveMonitor(value);
    }
    else
    {
        //----------------------#receive values by Float#
        //###FLOAT:TmcReceiveBlock()###
        rtn = m_connection.ReceiveBlockHeader(&maxLength);
        if (maxLength < 1)
        {
            return;
        }
        maxLength ++;//see tmctl's help
        if ((buf = new char[maxLength]) == NULL)
        {
            return;
        }

        int isEnd = 0;
        char bytes[sizeof(float)];
        float floatBuf;
        while (isEnd == 0)
        {
            memset(buf,0,maxLength);
            rtn = m_connection.ReceiveBlockData(buf, maxLength, &realLength, &isEnd);
            if (rtn != 0)
            {
                delete[] buf;
	            DispError(m_connection.GetLastError());
	            return;
            }
            for (n = 0; n < realLength; n ++)
            {
				data.Format(_T("%02X"), (unsigned char)buf[n]);
				value += data;
			}
			SetReceiveMonitor(value);
			
            //converse the char format and copy as float.
		    value = "";
            for (n = 0; n < realLength/4; n ++)
            {
                bytes[3] = buf[n * 4];
                bytes[2] = buf[n * 4 + 1];
                bytes[1] = buf[n * 4 + 2];
                bytes[0] = buf[n * 4 + 3];
                memcpy(&floatBuf, bytes, sizeof(float));
                data.Format(_T("%e"), floatBuf);
				value += data + ",";
           }
        }
        delete[] buf;
    }

    //----------------------#display data and output when saveBox checked#
    if (IsDlgButtonChecked(IDC_CHECK_SAVE) && !m_sFileName.IsEmpty())
    {
        ofstream csvFile(m_sFileName, ios::ate);
        if (csvFile.fail())
        {
            MessageBox("file can not open!", "Error", MB_ICONERROR | MB_OK);
            return;
        }

        //out put current time.
        SYSTEMTIME CurTime;
        GetLocalTime(&CurTime);
        csvFile << "Time: " << CurTime.wHour << ":" <<CurTime.wMinute << ":";
        //csvFile << CurTime.wSecond + CurTime.wMilliseconds * 0.001 << ",";
		CString msec;
		msec.Format("%d.%03d", CurTime.wSecond, CurTime.wMilliseconds);
        csvFile << (LPCSTR)msec << ",";

        for (n = 0; n < atoi(m_sComboItems); n ++)
        {
            //set display
            data = CutLeft(',', &value);
	        pEdit = (CEdit*)GetDlgItem(dLoop ++);
            pEdit->SetWindowText(data);
            //output data
            csvFile << data << ",";
        }
        csvFile << "\n";
        csvFile.close();
    }
    else
    {
        for (n = 0; n < atoi(m_sComboItems); n ++)
        {
            //set display
            data = CutLeft(',', &value);
	        pEdit = (CEdit*)GetDlgItem(dLoop ++);
            pEdit->SetWindowText(data);
        }
    }
}

//##########################################//
/// COMMUNICATIONS
//##########################################//
//==========================================//
/// <summary> Function: GetRanges </summary>
//==========================================//
bool CMainDialog::GetRanges(int eNo)
{
    int n;
    CString msg;
    CString terminal;
    CString module;

    m_comboU.ResetContent();
    m_comboI.ResetContent();
    //UpdateData(false); //used for DDX refresh

    //----------------------#Queries the Terminals#
    msg = ":INPUT:CURRENT:TERMINAL?";
    SetSendMonitor(msg);
    if (!QueriesData(150, msg, &terminal))
    {
        return false;
    }
    //find the terminal that is checked now
    for (n = 1; n < eNo; n++)
    {
        terminal = terminal.Mid(terminal.Find(";") + 1);
    }
    //Format:"INPUT:CURRENT:TERMINAL:ELEMENT1 DIRECT;ELEMENT2 SENSOR;..."
    terminal = CutLeft(';', &terminal);
    SetReceiveMonitor(terminal);

    //----------------------#Queries the Module#
    msg.Format(":INPUT:MODULE? %d", eNo);
    SetSendMonitor(msg);
    if (!QueriesData(150, msg, &module))
    {
        return false;
    }
    module = CutLeft((char)10, &module);
    SetReceiveMonitor(module);

    //----------------------#Set the Range Lists#
    //when crest factor == 3, set ranges.
    if (crestFactor == "3")
    {
        ///#set voltage list#
		n = 0;
        while (!voltageList[0][n].IsEmpty())
        {
            m_comboU.AddString(voltageList[0][n]);
            n++;
        }
        m_comboU.AddString(rangeListAuto);
        ///#set current list#
        if (module == "50")
        {
            // when 50A's terminal
            n = 6;
            while (!currentList[0][n].IsEmpty())
            {
                m_comboI.AddString(currentList[0][n]);
                n++;
            }
        }
        else if (module == "5")
        {
            // when 5A's terminal
            n = 0;
            while (!currentList[0][n + 3].IsEmpty())
            {
                m_comboI.AddString(currentList[0][n]);
                n++;
            }
        }
        n = 0;
        while (!currentList[2][n].IsEmpty())
        {
            m_comboI.AddString(currentList[2][n]);
            n++;
        }
        m_comboI.AddString(rangeListAuto);
    }
    //when crest factor == 6, set ranges.
    else if (crestFactor == "6")
    {
        ///#set voltage list#
        n = 0;
        while (!voltageList[1][n].IsEmpty())
        {
            m_comboU.AddString(voltageList[1][n]);
            n++;
        }
        m_comboU.AddString(rangeListAuto);
        ///#set current list#
        if (module == "50")
        {
            // when 50A's terminal
            n = 6;
            while (!currentList[1][n].IsEmpty())
            {
                m_comboI.AddString(currentList[1][n]);
                n++;
            }
        }
        else if (module == "5")
        {
            // when 5A's terminal
            n = 0;
            while (!currentList[1][n + 3].IsEmpty())
            {
                m_comboI.AddString(currentList[1][n]);
                n++;
            }
        }
        n = 0;
        while (!currentList[3][n].IsEmpty())
        {
            m_comboI.AddString(currentList[3][n]);
            n++;
        }
        m_comboI.AddString(rangeListAuto);
    }

    //----------------------#Get Voltage Range Settings#
    //###":VOLT:RANG:ELEM1 3.00E+00;ELEM2 1.00E+03"###
    CString range;
    double doubleRange;

    msg.Format(":INPUT:VOLTAGE:RANGE:ELEMENT%d?", eNo);
    SetSendMonitor(msg);
    if (!QueriesData(60, msg, &range))
    {
        return false;
    }
    SetReceiveMonitor(range);
    range = CutLeft((char)10, &range);//cut left with LF.

    if (range == rangeListAuto)
    {
        m_comboU.SetWindowText(range);
    }
    else
    {
        doubleRange = atof(range);
        range.Format(_T("%f"), doubleRange);
        range.TrimRight("0");
        range.TrimRight(".");
        m_comboU.SetWindowText(range + "V");
    }

    //----------------------#Get Current Range Settings#
    //###":CURR:RANG:ELEM5 3.00E+00;ELEM6 1.00E-03"###
    msg.Format(":INPUT:CURRENT:RANGE:ELEMENT%d?", eNo);
    SetSendMonitor(msg);
    if (!QueriesData(60, msg, &range))
    {
        return false;
    }
    SetReceiveMonitor(range);
    range = CutLeft((char)10, &range);//cut left with LF.

    if (range == rangeListAuto)
    {
        m_comboI.SetWindowText(range);
    }
	else
	{
        doubleRange = atof(range);
        if (doubleRange < 1)
        {
            //when "mA/mV" unit, multiply 1k.
            range.Format(_T("%f"), doubleRange * 1000);
            range.TrimRight("0");
            range.TrimRight(".");
            if (terminal.Find("DIR") != -1)
            {
                m_comboI.SetWindowText(range + "mA");
            }
            else //when using a sensor
            {
                m_comboI.SetWindowText(range + "mV");
            }
        }
        else
        {
            //###1.5V, 0.75V, 2.5A, ...###
            range.Format(_T("%f"), doubleRange);
            range.TrimRight("0");
            range.TrimRight(".");
            if (terminal.Find("DIR") != -1)
            {
                m_comboI.SetWindowText(range + "A");
            }
            else //when using a sensor
            {
                m_comboI.SetWindowText(range + "V");
            }
        }
    }
    return true;
}
//==========================================//
///E1~E6 Radio Check
//==========================================//
void CMainDialog::OnRadioE1()
{
	GetRanges(1);
}
void CMainDialog::OnRadioE2()
{
	GetRanges(2);
}
void CMainDialog::OnRadioE3()
{
	GetRanges(3);
}
void CMainDialog::OnRadioE4()
{
	GetRanges(4);
}
void CMainDialog::OnRadioE5()
{
	GetRanges(5);
}
void CMainDialog::OnRadioE6()
{
	GetRanges(6);
}

//==========================================//
///Set Range
//==========================================//
void CMainDialog::OnButtonSetRange() 
{
	CString range;
	CString msg;
    UpdateData(true);

	//----------------------#Send Voltage Range#
	m_comboU.GetWindowText(range);
	msg.Format(":INPUT:VOLTAGE:RANGE:ELEMENT%d %s", m_iRadioElement + 1, range);
	SetSendMonitor(msg);
    int rtn = m_connection.Send(msg);
    if (rtn != 0)
    {
	    DispError(m_connection.GetLastError());
	    //when setting failed, resume the original value.
	    GetRanges(m_iRadioElement + 1);
		return;
    }

	//----------------------#Send Current Range#
	//----------------------#Send Current Range#
	m_comboI.GetWindowText(range);
	if (range.Find("V") != -1)
	{
		msg.Format(":INPUT:CURRENT:TERMINAL:ELEMENT%d SENSOR", m_iRadioElement + 1);
	}
	else if (range.Find("A") != -1)
	{
		msg.Format(":INPUT:CURRENT:TERMINAL:ELEMENT%d DIRECT", m_iRadioElement + 1);
	}
	SetSendMonitor(msg);
	rtn = m_connection.Send(msg);
	if (rtn != 0)
	{
		DispError(m_connection.GetLastError());
	    //when setting failed, resume the original value.
	    GetRanges(m_iRadioElement + 1);
		return;
	}

	msg.Format(":INPUT:CURRENT:RANGE:ELEMENT%d %s", m_iRadioElement + 1, range);
	SetSendMonitor(msg);
	rtn = m_connection.Send(msg);
	if (rtn != 0)
	{
		DispError(m_connection.GetLastError());
	    //when setting failed, resume the original value.
	    GetRanges(m_iRadioElement + 1);
		return;
	}
	GetRanges(m_iRadioElement + 1);
}

//===========================================//
/// <summary>
/// Function: Get Update Rate
/// </summary>
//===========================================//
void CMainDialog::GetUpdateRate(CString value) 
{
    ///#get harmonics state#
    CString msg;
    ///#get harmonics state#
    if (value == "")
    {
        msg = ":HARMONICS:STATE?";
        SetSendMonitor(msg);
        if (!QueriesData(20, msg, &value))
        {
            return;
        }
        SetReceiveMonitor(value);
        value = CutLeft((char)10, &value);//cut left with LF.
    }

    ///#set updateRate combo list#
    m_comboR.ResetContent();
    CWnd* pWnd;
    if (value == "1")
    {
        pWnd = GetDlgItem(IDC_BUTTON_SETRATE);
        pWnd->EnableWindow(false);
        pWnd = GetDlgItem(IDC_COMBOR);
        pWnd->EnableWindow(false);
        return;
    }
    else
    {
        pWnd = GetDlgItem(IDC_BUTTON_SETRATE);
        pWnd->EnableWindow(true);
        pWnd = GetDlgItem(IDC_COMBOR);
        pWnd->EnableWindow(true);
	    int n = 0;
        while (!updateRateList[n].IsEmpty())
        {
            m_comboR.AddString(updateRateList[n]);
            n++;
        }
    }

    ///#get updateRate value#
    msg = ":RATE?";
    SetSendMonitor(msg);
    if (!QueriesData(30, msg, &value))
    {
        return;
    }
    SetReceiveMonitor(value);
    value = CutLeft((char)10, &value);//cut left with LF.

    double doubleRate = atof(value);
    if (doubleRate < 1)
    {
        //when "ms" unit, multiply 1k.
        value.Format(_T("%.0f"), doubleRate * 1000);
        m_comboR.SetWindowText(value + "ms");
    }
    else
    {
        value.Format(_T("%.0f"), doubleRate);
        m_comboR.SetWindowText(value + "s");
    }
}

//==========================================//
///Set UpdateRate
//==========================================//
void CMainDialog::OnButtonSetRate() 
{
    CString value;
    CString msg;
    ///#get harmonics state#
    msg = ":HARMONICS:STATE?";
    SetSendMonitor(msg);
    if (!QueriesData(20, msg, &value))
    {
        return;
    }
    SetReceiveMonitor(value);
    value = CutLeft((char)10, &value);//cut left with LF.

    if (value == "1")
    {
        m_comboR.ResetContent();
        DispError("Can not set update rate in harmonics mode!");
    }
    else
    {
	    //Send Command.
        m_comboR.GetWindowText(value);
        msg = ":RATE " + value;
	    SetSendMonitor(msg);
	    int rtn = m_connection.Send(msg);
        if (rtn != 0)
        {
	        DispError(m_connection.GetLastError());
            //when setting failed, resume the original value.
            GetUpdateRate(value);
        }
		GetUpdateRate(value);
    }
}

//==========================================//
///Test Command Send
//==========================================//
void CMainDialog::OnButtonSendCmd() 
{
    UpdateData(true);
    if (m_sEditTest == "")
    {
        return;
    }
	SetSendMonitor(m_sEditTest);
	if (m_sEditTest.Find("?") == -1)
    {
        int rtn;
	    //Send Command.
	    rtn = m_connection.Send(m_sEditTest);
        if (rtn != 0)
        {
	        DispError(m_connection.GetLastError());
        }
    }
    else
    {
        CString data;
        if (!QueriesData(1000, m_sEditTest, &data))
        {
            return;
        }
	    SetReceiveMonitor(data);
    }
}

//==========================================//
///Getting Error Information
//==========================================//
void CMainDialog::OnButtonGetError() 
{
    //----------------------#queries error from instrument#
    CString msg;
    CString errInfo;
    msg = ":STATUS:ERROR?";
    SetSendMonitor(msg);
    if (!QueriesData(150, msg, &errInfo))
    {
        return;
    }
    //cut off the return.
    errInfo = errInfo.Left(errInfo.Find((char)10));
    SetReceiveMonitor(errInfo);
    DispError(errInfo);
}

//==========================================//
///Set Header On/Off
//==========================================//
void CMainDialog::OnButtonHeader() 
{
    CString header;
    CString msg;
    //----------------------#queries header status#
    msg = ":COMMUNICATE:HEADER?";
    SetSendMonitor(msg);
    if (!QueriesData(25, msg, &header))
    {
        return;
    }
    SetReceiveMonitor(header);
    header = CutLeft((char)10, &header);//cut left with LF.

    //----------------------#set header status#
    if (header == "1")
    {
        msg = ":COMMUNICATE:HEADER OFF";
    }
    else
    {
        msg = ":COMMUNICATE:HEADER ON";
    }
    SetSendMonitor(msg);
    int rtn = m_connection.Send(msg);
    if (rtn != 0)
    {
	    DispError(m_connection.GetLastError());
	    return;
    }
}

//==========================================//
///Set Verbose On/Off
//==========================================//
void CMainDialog::OnButtonVerbose() 
{
    CString verbose;
    CString msg;
    //----------------------#queries verbose status#
    msg = ":COMMUNICATE:VERBOSE?";
    SetSendMonitor(msg);
    if (!QueriesData(25, msg, &verbose))
    {
        return;
    }
    SetReceiveMonitor(verbose);
    verbose = CutLeft((char)10, &verbose);//cut left with LF.

    //----------------------#set verbose status#
    if (verbose == "1")
    {
        msg = ":COMMUNICATE:VERBOSE OFF";
    }
    else
    {
        msg = ":COMMUNICATE:VERBOSE ON";
    }
    SetSendMonitor(msg);
    int rtn = m_connection.Send(msg);
    if (rtn != 0)
    {
	    DispError(m_connection.GetLastError());
	    return;
    }
}

//==========================================//
///System Reset
//==========================================//
void CMainDialog::OnButtonRst()
{
    int reset = MessageBox("System will be All Reset, continue?", "Confirm", MB_OKCANCEL|MB_ICONWARNING);
    if (reset == IDCANCEL)
    {
        return;
    }
    else
    {
        CString msg = "*RST";
	    SetSendMonitor(msg);
	    int rtn = m_connection.Send(msg);
        if (rtn != 0)
        {
	        DispError(m_connection.GetLastError());
	        return;
        }
        UpdateData();
        GetRanges(m_iRadioElement + 1);
        ReadItemSettings(0);//0: set normal/harmonics option according to the instrument.
    }
}

//=============================================//
///<summary>
///Function: ReadItemSettings
///</summary>
///<param>
///0(recheck), 1(normal), 2(harmonics)
///</param>
//=============================================//
void CMainDialog::ReadItemSettings(int dataType = 0) 
{
	CString msg;
    CString buf;
    CComboBox* pCombo;

    if (dataType == 0)
    {
        //----------------------#get NORMAL/HARMONICS#
        msg = ":HARMONICS:STATE?";
        SetSendMonitor(msg);
        if (!QueriesData(20, msg, &buf))
        {
            return;
        }
        SetReceiveMonitor(buf);
        buf = CutLeft((char)10, &buf);//cut left with LF.

        //----------------------#get update rate#
        GetUpdateRate(buf);

        //----------------------#set normal/harmonics option#
        if (buf == "1")
        {
            //check harmonics option
            CheckDlgButton(IDC_RADIO_NML, BST_UNCHECKED);
            CheckDlgButton(IDC_RADIO_HAR, BST_CHECKED);
            OnRadioHar();
            return;
        }
        else
        {
            //check normal option
            CheckDlgButton(IDC_RADIO_NML, BST_CHECKED);
            CheckDlgButton(IDC_RADIO_HAR, BST_UNCHECKED);
            OnRadioNml();
            return;
        }
    }

    //----------------------#get ASCII/BINARY#
    msg = ":NUMERIC:FORMAT?";
    SetSendMonitor(msg);
    if (!QueriesData(30, msg, &buf))
    {
        return;
    }
    SetReceiveMonitor(buf);
    buf = CutLeft((char)10, &buf);//cut left with LF.

    //----------------------#set ASCII/BINARY option#
    if (buf.Find("ASC") != -1)
    {
        CheckDlgButton(IDC_RADIO_ASC, BST_CHECKED);
        CheckDlgButton(IDC_RADIO_BIN, BST_UNCHECKED);
    }
    else
    {
        CheckDlgButton(IDC_RADIO_ASC, BST_UNCHECKED);
        CheckDlgButton(IDC_RADIO_BIN, BST_CHECKED);
    }

    //----------------------#get item count#
    if (dataType == 1)//IsDlgButtonChecked(IDC_RADIO_NML))
    {
        msg = ":NUMERIC:NORMAL:NUMBER?";
    }
    else
    {
        msg = ":NUMERIC:HARMONICS:NUMBER?";
    }
    SetSendMonitor(msg);
    if (!QueriesData(30, msg, &buf))
    {
        return;
    }
    SetReceiveMonitor(buf);
    buf = CutLeft((char)10, &buf);//cut left with LF.

    //----------------------#set item count combo#
    pCombo = (CComboBox*)GetDlgItem(IDC_COMBO_ITEMS);
    if (atoi(buf) > MAX_ITEM)
    {
        CString maxItem;
        maxItem.Format(_T("%d"), MAX_ITEM);
        ComboListSel(pCombo, maxItem);
    }
    else
    {
        ComboListSel(pCombo, buf);
    }
    OnSelchangeComboItems();

    //----------------------#get item settings#
    if (dataType == 1)//IsDlgButtonChecked(IDC_RADIO_NML))
    {
        msg = ":NUMERIC:NORMAL?";
    }
    else
    {
        msg = ":NUMERIC:HARMONICS?";
    }
    //###:NUM:NUM 6;ITEM IMN,6;ITEM2 IDC,5;ITEM3 IAC,4;ITEM4 P,3;ITEM5 S,2;ITEM6 URMS,1###
    SetSendMonitor(msg);
    if (!QueriesData(30 + 30*atoi(buf), msg, &buf))
    {
        return;
    }
    SetReceiveMonitor(buf);
    buf = CutLeft((char)10, &buf);//cut left with LF.
    //cut off the item number(item count) portion.
    CutLeft(';', &buf);

    //display as "LAMB" when received a "LAMBDA"(Functions).
    buf.Replace("LAMB,", "LAMBDA,");
    buf.Replace("PP,", "PPEAK,");
    buf.Replace("MP,", "MPEAK,");
    buf.Replace("TORQ,", "TORQUE,");
    buf.Replace("SPE,", "SPEED,");
    buf.Replace("ET,", "ETA,");
    //display as "SGM" when received a "SIGM"(Elements).
    buf.Replace("SIGM", "SGM");

    //----------------------#set item settings to be display#
    UpdateData(true);
    CString itemBuf;
    CString element;
    int fLoop = IDC_COMBO_F1;
    int eLoop = IDC_COMBO_E1;
    int oLoop = IDC_COMBO_O1;
    int n;

    for (n = 0; n < atoi(m_sComboItems); n ++)
    {
        itemBuf = CutLeft(';', &buf);
        //#set function.#
	    pCombo = (CComboBox*)GetDlgItem(fLoop ++);
        pCombo->SetWindowText(CutLeft(',', &itemBuf));
        //#set element.#
	    pCombo = (CComboBox*)GetDlgItem(eLoop ++);
        element = CutLeft(',', &itemBuf);
        if (element == "SGM")
        {
            //display as "SGMA"
            element = "SGMA";
        }
        pCombo->SetWindowText(element);
        
        //#set order when harmonics.#
        if (dataType == 2)//!IsDlgButtonChecked(IDC_RADIO_NML))
        {
            if (itemBuf.Find("TOT") != -1)
            {
                //display "TOT" as "TOTAL"
                itemBuf = "TOTAL";
            }
	        pCombo = (CComboBox*)GetDlgItem(oLoop ++);
            pCombo->SetWindowText(itemBuf);
        }
    }
}

//==========================================//
///<summary>
///Function: SendItemSettings
///</summary>
//==========================================//
void CMainDialog::SendItemSettings() 
{
    CString msg;
    CString state;
    int rtn;

    //----------------------#get Normal/Harmonics' setting#
    msg = ":HARMONICS:STATE?";
    //###:HARMONICS:STATE 1/ :HARM 1/ 1###
    SetSendMonitor(msg);
    if (!QueriesData(20, msg, &state))
    {
        return;
    }
    SetReceiveMonitor(state);
    state = CutLeft((char)10, &state);//cut left with LF.

    //----------------------#send Normal/Harmonics' setting#
    msg = "";
    if (IsDlgButtonChecked(IDC_RADIO_NML) && state == "1")
    {
        msg = ":HARMONICS:STATE OFF";
        state = "0";
    }
    else if (IsDlgButtonChecked(IDC_RADIO_HAR) && state == "0")
    {
        msg = ":HARMONICS:STATE ON";
        state = "1";
    }
    if (msg != "")
    {
        SetSendMonitor(msg);
        rtn = m_connection.Send(msg);
        if (rtn != 0)
        {
	        DispError(m_connection.GetLastError());
	        return;
        }
    }

    //----------------------#get UpdateRate#
    //when normal/harmonics changed,
    //refresh updateRate displays.
    GetUpdateRate(state);

    //----------------------#set ASCII/Float(Binary)#
    if (IsDlgButtonChecked(IDC_RADIO_ASC))
    {
        msg = ":NUMERIC:FORMAT ASCII";
    }
    else
    {
        msg = ":NUMERIC:FORMAT FLOAT";
    }
    SetSendMonitor(msg);
    rtn = m_connection.Send(msg);
    if (rtn != 0)
    {
	    DispError(m_connection.GetLastError());
	    return;
    }

    //----------------------#set items number#
    if (IsDlgButtonChecked(IDC_RADIO_NML))
    {
        msg = ":NUMERIC:NORMAL:NUMBER " + m_sComboItems;
    }
    else
    {
        msg = ":NUMERIC:HARMONICS:NUMBER " + m_sComboItems;
    }
    SetSendMonitor(msg);
    rtn = m_connection.Send(msg);
    if (rtn != 0)
    {
	    DispError(m_connection.GetLastError());
	    return;
    }

    //----------------------#send message detail#
    int n;
    CString itemIndex;
    CString param;
    CComboBox* pCombo;
    int fLoop = IDC_COMBO_F1;
    int eLoop = IDC_COMBO_E1;
    int oLoop = IDC_COMBO_O1;

    if (IsDlgButtonChecked(IDC_RADIO_NML))
    {
        msg = ":NUMERIC:NORMAL:";
    }
    else
    {
        msg = ":NUMERIC:HARMONICS:";
    }
    for (n = 0; n < atoi(m_sComboItems); n ++)
    {
        itemIndex.Format(_T("%d"), n + 1);

        //set function parameter into message.
        pCombo = (CComboBox*)GetDlgItem(fLoop ++);
        pCombo->GetWindowText(param);
        msg = msg + "ITEM" + itemIndex + " " + param;

        //set element parameter into message.
        pCombo = (CComboBox*)GetDlgItem(eLoop ++);
        pCombo->GetWindowText(param);
        msg = msg + "," + param;

        //set order parameter into message, if have.
        pCombo = (CComboBox*)GetDlgItem(oLoop ++);
        pCombo->GetWindowText(param);
        if (param != "")
        {
            msg = msg + "," + param;
        }

        //set separator into message.
        if (n != atoi(m_sComboItems) - 1)
        {
            msg = msg + ";";
        }
    }
    msg.Replace("SGM", "SIGM");//sending as "SIGM" when display as "SGM".

    SetSendMonitor(msg);
    rtn = m_connection.Send(msg);
    if (rtn != 0)
    {
	    DispError(m_connection.GetLastError());
	    return;
    }
}

//=============================================//
///Read Item Settings
///dataType: 0(recheck), 1(normal), 2(harmonics)
//=============================================//
void CMainDialog::OnButtonGetItems() 
{
    ReadItemSettings(0);
}

//==========================================//
///Send Item Settings
//==========================================//
void CMainDialog::OnButtonSetItems() 
{
    SendItemSettings();
}

//==========================================//
///<summary>
///Function: EnableItems
///</summary>
//==========================================//
void CMainDialog::EnableItems(bool isEnable)
{
    GetDlgItem(IDC_BUTTON_SETRATE)->EnableWindow(isEnable);
    GetDlgItem(IDC_COMBOR)->EnableWindow(isEnable);
    GetDlgItem(IDC_BUTTON_SETRANGE)->EnableWindow(isEnable);
    GetDlgItem(IDC_BUTTON_SENDCMD)->EnableWindow(isEnable);
    GetDlgItem(IDC_BUTTON_GETERR)->EnableWindow(isEnable);
    GetDlgItem(IDC_BUTTON_RST)->EnableWindow(isEnable);
    GetDlgItem(IDC_BUTTON_HEADER)->EnableWindow(isEnable);
    GetDlgItem(IDC_BUTTON_VERBOSE)->EnableWindow(isEnable);

    GetDlgItem(IDC_BUTTON_GETITEMS)->EnableWindow(isEnable);
    GetDlgItem(IDC_BUTTON_SETITEMS)->EnableWindow(isEnable);
    GetDlgItem(IDC_COMBO_ITEMS)->EnableWindow(isEnable);
    GetDlgItem(IDC_RADIO_NML)->EnableWindow(isEnable);
    GetDlgItem(IDC_RADIO_HAR)->EnableWindow(isEnable);
    GetDlgItem(IDC_RADIO_ASC)->EnableWindow(isEnable);
    GetDlgItem(IDC_RADIO_BIN)->EnableWindow(isEnable);

    GetDlgItem(IDC_BUTTON_GETDATA_S)->EnableWindow(isEnable);
    GetDlgItem(IDC_BUTTON_GETDATA_R)->EnableWindow(isEnable);
    GetDlgItem(IDC_BUTTON_GETDATA_T)->EnableWindow(isEnable);
    GetDlgItem(IDC_EDIT_TMR)->EnableWindow(isEnable);
    GetDlgItem(IDC_CHECK_SAVE)->EnableWindow(isEnable);
    GetDlgItem(IDC_EDIT_FILE)->EnableWindow(isEnable);
}

//==========================================//
///<summary>
///Timer Event
///</summary>
//==========================================//
void CMainDialog::OnTimer(UINT nIDEvent)
{
    if (nIDEvent == 1)//get data by update rate.
    {
        CString eesr;
        CString msg = ":STATUS:EESR?";
        SetSendMonitor(msg);
        if (!QueriesData(20, msg, &eesr))
        {
            return;
        }
        SetReceiveMonitor(eesr);
        eesr = CutLeft((char)10, &eesr);//cut left with LF.

        if (atoi(eesr) & 0X01 == 1)
        {
            GetItemData();
        }
    }
    else if (nIDEvent == 2)//get data by timer.
    {
        GetItemData();
        return;
    }
}

//==========================================//
///GetData Single
//==========================================//
void CMainDialog::OnButtonGetDataS() 
{
    SendItemSettings();
    GetItemData();
}

//==========================================//
///GetData by UpdateRate
//==========================================//
void CMainDialog::OnButtonGetDataR() 
{
    CString caption;
    CWnd* pWnd;
    pWnd = GetDlgItem(IDC_BUTTON_GETDATA_R);
    pWnd->GetWindowText(caption);
    //----------------------#resume all#
    if (caption == "STOP")
    {
        KillTimer(TIMER_1);
        pWnd->SetWindowText("    Get Data      (Update Rate)");
        EnableItems(true);
        //resume fileNameInputText.
        if (!IsDlgButtonChecked(IDC_CHECK_SAVE))
        {
            pWnd = (CButton*)GetDlgItem(IDC_EDIT_FILE);
            pWnd->EnableWindow(false);
        }
        //resume updateRate.
        if (!IsDlgButtonChecked(IDC_RADIO_NML))
        {
            pWnd = (CButton*)GetDlgItem(IDC_COMBOR);
            pWnd->EnableWindow(false);
            pWnd = (CButton*)GetDlgItem(IDC_BUTTON_SETRATE);
            pWnd->EnableWindow(false);
        }
    }
    //----------------------#getting datas#
    else
    {
        UpdateData(true);
        if (IsDlgButtonChecked(IDC_CHECK_SAVE) && !m_sFileName.IsEmpty())
        {
            //print titles(function) once when saveBox is checked
            ofstream csvFile(m_sFileName, ios::ate);
            if (csvFile.fail())
            {
                MessageBox("file can not open!", "Error", MB_ICONERROR | MB_OK);
                return;
            }

            //out put current time.
            SYSTEMTIME CurTime;
            GetLocalTime(&CurTime);
            csvFile << "Date: " << CurTime.wYear << "/" << CurTime.wMonth;
            csvFile << "/" << CurTime.wDay << ",";

            int fLoop = IDC_COMBO_F1;
            int n;
            CString function;
            CComboBox* pCombo;
            for (n = 0; n < atoi(m_sComboItems); n ++)
            {
	            pCombo = (CComboBox*)GetDlgItem(fLoop ++);
                pCombo->GetWindowText(function);
                csvFile << function << ",";
            }
            csvFile << "\n";
            csvFile.close();
        }
        SendItemSettings();

        //reset filter1.
        CString msg = ":STATUS:FILTER1 FALL";
        SetSendMonitor(msg);
	    int rtn = m_connection.Send(msg);
        if (rtn != 0)
        {
	        DispError(m_connection.GetLastError());
	        return;
        }

        //clear eesr.
        CString eesr;
        msg = ":STATUS:EESR?";
        SetSendMonitor(msg);
        if (!QueriesData(20, msg, &eesr))
        {
            return;
        }
        SetReceiveMonitor(eesr);

        //reset other controls' display.
        EnableItems(false);
        pWnd->EnableWindow(true);
        pWnd->SetWindowText("STOP");
        
        //set timer interval and start getting data.
        CWnd::SetTimer(TIMER_1,20,NULL);
    }
}

//==========================================//
///GetData by Timer
//==========================================//
void CMainDialog::OnButtonGetDataT() 
{
    CString text;
    CWnd* pWnd;
    pWnd = GetDlgItem(IDC_BUTTON_GETDATA_T);
    pWnd->GetWindowText(text);
    //----------------------#resume all#
    if (text == "STOP")
    {
        KillTimer(TIMER_2);
        pWnd->SetWindowText("Get Data (Timer)");
        EnableItems(true);
        //resume fileNameInputText.
        if (!IsDlgButtonChecked(IDC_CHECK_SAVE))
        {
            pWnd = (CButton*)GetDlgItem(IDC_EDIT_FILE);
            pWnd->EnableWindow(false);
        }
        //resume updateRate.
        if (!IsDlgButtonChecked(IDC_RADIO_NML))
        {
            pWnd = (CButton*)GetDlgItem(IDC_COMBOR);
            pWnd->EnableWindow(false);
            pWnd = (CButton*)GetDlgItem(IDC_BUTTON_SETRATE);
            pWnd->EnableWindow(false);
        }
    }
    //----------------------#getting datas#
    else
    {

        if (IsDlgButtonChecked(IDC_CHECK_SAVE) && !m_sFileName.IsEmpty())
        {
            //print titles(function) once while saveBox is checked.
            ofstream csvFile(m_sFileName, ios::ate);
            if (csvFile.fail())
            {
                MessageBox("file can not open!", "Error", MB_ICONERROR | MB_OK);
                return;
            }

            //out put current time.
            SYSTEMTIME CurTime;
            GetLocalTime(&CurTime);
            csvFile << "Date: " << CurTime.wYear << "/" << CurTime.wMonth;
            csvFile << "/" << CurTime.wDay << ",";

            int fLoop = IDC_COMBO_F1;
            int n;
            CString function;
            CComboBox* pCombo;
            for (n = 0; n < atoi(m_sComboItems); n ++)
            {
	            pCombo = (CComboBox*)GetDlgItem(fLoop ++);
                pCombo->GetWindowText(function);
                csvFile << function << ",";
            }
            csvFile << "\n";
            csvFile.close();
        }

        SendItemSettings();
        //reset other controls' display.
        EnableItems(false);
        pWnd->EnableWindow(true);
        pWnd->SetWindowText("STOP");

        //set timer interval and start getting data.
        pWnd = GetDlgItem(IDC_EDIT_TMR);
        pWnd->GetWindowText(text);
        int timer = atoi(text);
        if (timer < 20)
        {
            timer = 20;
            pWnd->SetWindowText("20");
        }
        CWnd::SetTimer(TIMER_2, timer, NULL);
    }
}

//##########################################//
// DISPLAY CONTROL
//##########################################//

//==========================================//
///Normal Checked
//==========================================//
void CMainDialog::OnRadioNml() 
{
    int itemIndex;
    int n;
	CComboBox* pCombo;
	CEdit* pEdit;
    int fLoop = IDC_COMBO_F1;
    int eLoop = IDC_COMBO_E1;
    int oLoop = IDC_COMBO_O1;
    int dLoop = IDC_EDIT_D1;

    CString opt;
    CString msg = "*OPT?";
    if (!QueriesData(20, msg, &opt))
    {
        return;
    }
    for (itemIndex = 0; itemIndex < MAX_ITEM; itemIndex ++)
    {
        //----------------------#set function comboBoxes#
	    pCombo = (CComboBox*)GetDlgItem(fLoop ++);
        pCombo->ResetContent();
        n = 0;
        while (!nList[n].IsEmpty())
        {
            //when not motor option, do not add motor functions(38~44).
            if (n > 37 && n < 45 && opt.Find("MTR") == -1)
            {
                n ++;
                continue;
            }
            pCombo->AddString(nList[n]);
            n ++;
        }
        pCombo->SetCurSel(0);

        //----------------------#set elememt comboBoxes#
	    pCombo = (CComboBox*)GetDlgItem(eLoop ++);
        pCombo->ResetContent();
        for (n = 0; n < lastElement; n ++)
        {
            pCombo->AddString(eList[n]);
        }
        int m;
        if (lastElement == 1)
        {
            m = 0;//when 1 element only, invalidate SIGMA.
        }
        else
        {
            //when 3~4/5~6 element, SGM A,B/A,B,C can be use.
            m = (lastElement - 1) / 2 + 1;
        }
        for ( n = MAX_ELEMENT; n < MAX_ELEMENT + m; n ++)
        {
            pCombo->AddString(eList[n]);
        }
        pCombo->SetCurSel(0);

        //----------------------#set order comboBoxes#
	    pCombo = (CComboBox*)GetDlgItem(oLoop ++);
        pCombo->ResetContent();
		pCombo->EnableWindow(false);

        //----------------------#set data editBoxes#
	    pEdit = (CEdit*)GetDlgItem(dLoop ++);
        pEdit->SetWindowText("");
    }
    //----------------------#get settings from instrument#
    ReadItemSettings(1);
}

//==========================================//
///Harmonics Checked
//==========================================//
void CMainDialog::OnRadioHar() 
{
    int itemIndex;
    int n;
	CComboBox* pCombo;
	CEdit* pEdit;
    int fLoop = IDC_COMBO_F1;
    int eLoop = IDC_COMBO_E1;
    int oLoop = IDC_COMBO_O1;
    int dLoop = IDC_EDIT_D1;

    for (itemIndex = 0; itemIndex < MAX_ITEM; itemIndex ++)
    {
        //----------------------#set function comboBoxes#
	    pCombo = (CComboBox*)GetDlgItem(fLoop ++);
        pCombo->ResetContent();
        n = 0;
        while (!hList[n].IsEmpty())
        {
            pCombo->AddString(hList[n]);
            n++;
        }
        pCombo->SetCurSel(0);

        //----------------------#set elememt comboBoxes#
	    pCombo = (CComboBox*)GetDlgItem(eLoop ++);
        pCombo->ResetContent();
        for (n = 0; n < lastElement; n ++)
        {
            pCombo->AddString(eList[n]);
        }
        int m;
        if (lastElement == 1)
        {
            m = 0;//when 1 element only, invalidate SIGMA.
        }
        else
        {
            //when 3~4/5~6 element, SIGMA A,B/A,B,C can be use.
            m = (lastElement - 1) / 2 + 1;
        }
        for ( n = MAX_ELEMENT; n < MAX_ELEMENT + m; n ++)
        {
            pCombo->AddString(eList[n]);
        }
        pCombo->SetCurSel(0);

        //----------------------#set order comboBoxes#
	    pCombo = (CComboBox*)GetDlgItem(oLoop ++);
        if (itemIndex < atoi(m_sComboItems))
        {
		    pCombo->EnableWindow(true);
        }
        pCombo->ResetContent();
        n = 0;
        while (!oList[n].IsEmpty())
        {
            pCombo->AddString(oList[n]);
            n++;
        }
        pCombo->SetCurSel(0);

        //----------------------#set data editBoxes#
	    pEdit = (CEdit*)GetDlgItem(dLoop ++);
        pEdit->SetWindowText("");
    }
    //----------------------#get settings from instrument#
    ReadItemSettings(2);
}

//==========================================//
///DataSave Box Checked
//==========================================//
void CMainDialog::OnCheckSave() 
{
    CWnd* pWnd = GetDlgItem(IDC_EDIT_FILE);
    if (IsDlgButtonChecked(IDC_CHECK_SAVE))
    {
        pWnd->EnableWindow(true);
    }
    else
    {
        pWnd->EnableWindow(false);
    }
}

//==========================================//
///Items Count Changed
//==========================================//
void CMainDialog::OnSelchangeComboItems()
{
    int n;
	CWnd* pWndCtrl;
	//CEdit* pEdit;
    UpdateData();

	//----------------------#set to ReadOnly(true/false)#
    int fLoop = IDC_COMBO_F1;
    int eLoop = IDC_COMBO_E1;
    int oLoop = IDC_COMBO_O1;
    int dLoop = IDC_EDIT_D1;
	for ( n = 0; n < atoi(m_sComboItems); n ++)
	{
	    GetDlgItem(fLoop ++)->EnableWindow(true);
	    GetDlgItem(eLoop ++)->EnableWindow(true);
	    pWndCtrl = GetDlgItem(oLoop ++);
        if (!IsDlgButtonChecked(IDC_RADIO_NML))
        {
		    pWndCtrl->EnableWindow(true);
        }
	    GetDlgItem(dLoop ++)->EnableWindow(true);
	}
	while (n < MAX_ITEM)
	{
	    pWndCtrl = GetDlgItem(fLoop ++);
		pWndCtrl->EnableWindow(false);
	    pWndCtrl = GetDlgItem(eLoop ++);
		pWndCtrl->EnableWindow(false);
	    pWndCtrl = GetDlgItem(oLoop ++);
		pWndCtrl->EnableWindow(false);
	    pWndCtrl = GetDlgItem(dLoop ++);
		pWndCtrl->EnableWindow(false);
		n ++;
	}
}

//==========================================//
///Clear Send Monitor
//==========================================//
void CMainDialog::OnButtonSclr() 
{
    CWnd* pWnd = GetDlgItem(IDC_EDIT_SEND);
    pWnd->SetWindowText("");
}

//==========================================//
///Clear Receive Monitor
//==========================================//
void CMainDialog::OnButtonRclr() 
{
    CWnd* pWnd = GetDlgItem(IDC_EDIT_RCV);
    pWnd->SetWindowText("");
}
