/*STARTM-----------------------------------------------------------------
+       Module Name     : tmctl.h                                       +
+       Abstract        : Yokogawa T&M Remote Control functions.        +
+       Revision History:                                               +
+       Rev.    Date.          Notes.                                   +
+       --------------------------------------------------------------  +
+       0       2001.04.01                                              +
-ENDM-------------------------------------------------------------------*/

#ifndef	_TMCTL_H
#define	_TMCTL_H	1

/* Control setting */
#define	TM_NO_CONTROL	0
#define	TM_CTL_GPIB		1
#define	TM_CTL_RS232	2
#define	TM_CTL_USB		3
#define	TM_CTL_ETHER	4

/* GPIB */


/* RS232 */
#define	TM_RS_1200	'0'
#define	TM_RS_2400	'1'
#define	TM_RS_4800	'2'
#define	TM_RS_9600	'3'
#define	TM_RS_19200	'4'
#define	TM_RS_38400	'5'
#define	TM_RS_57600	'6'

#define	TM_RS_8N	'0'
#define	TM_RS_7E	'1'
#define	TM_RS_7O	'2'

#define	TM_RS_NO	'0'
#define	TM_RS_XON	'1'
#define	TM_RS_HARD	'2'


/* USB */

/* Error Number */
#define	TMCTL_NO_ERROR				0x00000000		/* No error */
#define	TMCTL_TIMEOUT				0x00000001		/* Timeout */
#define	TMCTL_NO_DEVICE				0x00000002		/* Device Not Found */
#define	TMCTL_FAIL_OPEN				0x00000004		/* Open Port Error */
#define	TMCTL_NOT_OPEN				0x00000008		/* Device Not Open */
#define	TMCTL_DEVICE_ALREADY_OPEN	0x00000010		/* Device Already Open */
#define	TMCTL_NOT_CONTROL			0x00000020		/* Controller Not Found */
#define	TMCTL_ILLEGAL_PARAMETER		0x00000040		/* Parameter is illegal */
#define	TMCTL_SEND_ERROR			0x00000100		/* Send Error */
#define	TMCTL_RECV_ERROR			0x00000200		/* Receive Error */
#define	TMCTL_NOT_BLOCK				0x00000400		/* Data is not Block Data */
#define	TMCTL_SYSTEM_ERROR			0x00001000		/* System Error */
#define	TMCTL_ILLEGAL_ID			0x00002000		/* Device ID is Illegal */


/* Functions */
#ifndef	_TMCTL_DEFINES

#ifdef	__cplusplus

extern	"C" {
#endif
extern	int		__stdcall TmcInitialize( int wire, char* adr, int* id );
extern	int		__stdcall TmcSetIFC( int id, int tm );
extern	int		__stdcall TmcDeviceClear( int id );

extern	int		__stdcall TmcSend( int id, char* msg );
extern	int		__stdcall TmcSendByLength( int id, char* msg, int len );
extern	int		__stdcall TmcSendSetup( int id );
extern	int		__stdcall TmcSendOnly( int id, char* msg, int len, int end );

extern	int		__stdcall TmcReceive( int id, char* buff, int blen, int* rlen );
extern	int		__stdcall TmcReceiveSetup( int id );
extern	int		__stdcall TmcReceiveOnly( int id, char* buff, int blen, int* rlen );
extern	int		__stdcall TmcReceiveBlockHeader( int id, int* len );
extern	int		__stdcall TmcReceiveBlockData( int id, char* buff, int blen, int* rlen, int* end );

extern	int		__stdcall TmcCheckEnd( int id );
extern	int		__stdcall TmcSetCmd( int id, char* cmd );
extern	int		__stdcall TmcSetRen( int id, int flag );
extern	int		__stdcall TmcGetLastError( int id );
extern	int		__stdcall TmcGetDetailLastError( int id );
extern	int		__stdcall TmcCheckError( int id, int sts, char* msg, char* err );
extern	int		__stdcall TmcSetTerm( int id, int eos, int eot );
extern	int		__stdcall TmcSetEos( int id, unsigned char eos );
extern	int		__stdcall TmcSetTimeout( int id, int tmo );
extern	int		__stdcall TmcSetDma( int id, int flg );
extern	int		__stdcall TmcGetStatusByte( int id, unsigned char* sts );
extern	int		__stdcall TmcFinish( int id );
#ifdef	__cplusplus
}

#endif
#endif
#endif


